"""project_users table

Revision ID: 401b70052c48
Revises: 33fd38e158e0
Create Date: 2019-03-09 18:41:25.371331

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '401b70052c48'
down_revision = '33fd38e158e0'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('projects_users_',
    sa.Column('project_id', sa.Integer(), nullable=True),
    sa.Column('user_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['project_id'], ['projects.id'], ),
    sa.ForeignKeyConstraint(['user_id'], ['users.id'], )
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('projects_users_')
    # ### end Alembic commands ###
